package nl.amis.model.library;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.SQLException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


import nl.amis.model.library.common.LibraryService;

import oracle.jbo.JboException;
import oracle.jbo.domain.DBSequence;
import oracle.jbo.server.ApplicationModuleImpl;

import oracle.jdbc.OracleCallableStatement;
import oracle.jdbc.OracleTypes;

import oracle.sql.ARRAY;
import oracle.sql.ArrayDescriptor;
import oracle.sql.STRUCT;
import oracle.sql.StructDescriptor;
// ---------------------------------------------------------------------
// ---    File generated by Oracle ADF Business Components Design Time.
// ---    Custom code may be added to this class.
// ---    Warning: Do not modify method signatures of generated methods.
// ---------------------------------------------------------------------
public class LibraryServiceImpl extends ApplicationModuleImpl implements LibraryService {
    /**This is the default constructor (do not remove)
     */
    public LibraryServiceImpl() {
    }


    public List<Map> getBooks(String author) {
        List<Map> books = new ArrayList();
        String getBooksStatement = 
            "begin ? := LIBRARIAN.get_books_by_author(?); end;";
        ARRAY nt = null;

        CallableStatement st = null;
        try {
            st = 
 getDBTransaction().createCallableStatement(getBooksStatement, 0);
            // Register the first bind variable for the return value
            st.registerOutParameter(1, OracleTypes.ARRAY, "SHELF");
            st.setString(2, author);
            st.executeUpdate();
            // 6. Return the value of the first bind variable
            nt = ((OracleCallableStatement)st).getARRAY(1);
            // the ARRAY object can yield an array of objects
            Object[] booksArray = (Object[])nt.getArray();
            // the objects returned in the ARRAY are in fact STRUCTs
            // these are created based on the BOOK SQL TYPE definition
            // each of the properties in the BOOK TYPE correspond to a
            // an attribute in the STRUCT
            for (int i = 0; i < booksArray.length; i++) {
                STRUCT bookStruct = (STRUCT)booksArray[i];
                Map book = new HashMap();
                book.put("TITLE", bookStruct.getAttributes()[0]);
                book.put("AUTHOR", bookStruct.getAttributes()[1]);
                book.put("ISBN", bookStruct.getAttributes()[2]);
                book.put("YEAR_OF_PUBLICATION", bookStruct.getAttributes()[3]);
                books.add(book);
            } // for
        } catch (SQLException e) {
            throw new JboException(e);
        } finally {
            if (st != null) {
                try {
                    st.close();
                } catch (SQLException e) {
                    throw new JboException(e);
                }
            }
        }
        return books;
    }

    public void saveBooks(List<Map> books) {
        Connection conn;

        // create the StructureDescriptor to the Database Type NGN_SET_MEMBER_T

        // A StructDescriptor can be thought of as a "type object". This means that it contains information about the object type, including the typecode, the type name, and how to convert to and from the given type
        // To construct an oracle.sql.STRUCT object for a given Oracle object type, you must:
        //    1. Create a StructDescriptor object (if one does not already exist) for the given Oracle object type.
        //    2. Use the StructDescriptor to construct the STRUCT object.
        // A StructDescriptor is an instance of the oracle.sql.StructDescriptor class and describes a type of SQL structured object (Oracle object).
        try {
            conn = getDBTransaction().createStatement(1).getConnection();
            StructDescriptor bookStructDesc = 
                StructDescriptor.createDescriptor("BOOK", conn);

            // the array that will contain the Struct objects - one object for everyt member
            Object[] bookStructArray = new Object[books.size()];

            // for each member in List members
            // create a Struct object and add to the memberArray
            int ctr = 0;
            for (Map book: books) {

                STRUCT m = 
                    new STRUCT(bookStructDesc, conn, new Object[] { book.get("TITLE"), 
                                                                    book.get("AUTHOR"), 
                                                                    book.get("ISBN"), 
                                                                    book.get("YEAR_OF_PUBLICATION") });
                bookStructArray[ctr++] = m;
            }


            // create the structure array descriptor for the NGN_SET_MEMBER_TBL_T type

            ArrayDescriptor shelfStructDesc = 
                ArrayDescriptor.createDescriptor("SHELF", conn);
            // using the memberListStructDesc descriptor we can create an ARRAY object that contains the memberArray (filled with memberStruct objects)
            ARRAY shelfStruct = 
                new ARRAY(shelfStructDesc, conn, bookStructArray);

            String saveBooksStatement = 
                "begin librarian.add_entire_shelf_to_library(?); end;";


            OracleCallableStatement st = null;
            try {
                st = 
 (OracleCallableStatement)getDBTransaction().createCallableStatement(saveBooksStatement, 
                                                                     0);
                st.setARRAY(1, shelfStruct);
                st.executeUpdate();


            } catch (SQLException e) {
                throw new JboException(e);
            } finally {
                if (st != null) {
                    try {
                        st.close();
                    } catch (SQLException e) {
                        throw new JboException(e);
                    }
                }
            }
        } catch (SQLException e) {
            // TODO
        }


    }

    /**Sample main for debugging Business Components code using the tester.
     */
    public static void main(String[] args) { /* package name */
        /* Configuration Name */launchTester("nl.amis.model.library", 
                                             "LibraryServiceLocal");
    }
}
